#pragma once
#ifndef CHANNEL_4_H
#define CHANNEL_4_H

// Basisklasse einbinden
#include "datafox/communication/module_table.h"

// Klasse für den Datenspeicher einbinden.
#include "channel_4_buffer.h"

// defines fuer Setup
#include "file/interface/studio_setup_structs.h"

// Geraetedateiarchiv einbinden
#include "file/file_device_zip.h"

// Studiodatei einbinden
#include "file/file_studio_setup.h"

#include "file/file_hardware_info.h"

#include "common/device_info.h"
#include "common/tcpip_info.h"

#include <string>

#include <thread>
#include <mutex>

// Defines setzen, wenn nicht bereits geschehen
#if !defined(_MSC_VER) && !defined(__MINGW32__)

#ifndef WINAPI
#define WINAPI __stdcall
#endif

typedef void *LPVOID;
typedef unsigned int DWORD;

#endif

#if defined(_MSC_VER) || defined(__MINGW32__)
#ifndef S_ISREG
#define S_ISREG(m) ((m & _S_IFMT) == _S_IFREG)
#endif

#ifndef S_ISDIR
#define S_ISDIR(m) ((m & _S_IFMT) == _S_IFDIR)
#endif
#endif

#ifndef STRUCT_MEM_FILE_HEADER
#define STRUCT_MEM_FILE_HEADER
struct StructMemFileHeader
{
    int structSize;          // Strukturgroesse
    int type;                // Speichertyp
    int baseAddress;         // Basisadresse
    int memAddress;          // Anfangsadresse
    int memSize;
    char tmp[76];
    //...
    int crcHead;    // 32bit CRC fuer Head (Immer letzter Wert in der Struktur; 
    // Berechnungsmenge: (sizeof(SDFFileDeviceHeader) - sizeof(unsigned short)), also ohne sich selbst!)
};
#endif

// Struktur für die Datei der Fingertemplates
//############################################
// Innerer Aufbau der Datei ist wie folgt:
// StructFTFileHeader, (count * [(type == 0) ? structFingertemplateDIN : structFingertemplateIdencom, CRC16 ])
// - Zuerst liegt die Headerstruktur StructFTFileHeader in der Datei.
// - Der erste 16bit-Wert der Datei muss 0xDFDF sein.
// - Der crc der Headerstruktur berechnet sich über die Struktur ohne die zwei Byte des CRC selbst.
// - Die Daten beginnen ab der Position Seek(StructFTFileHeader.offs, Filebegin).
// - Der Datenteil besteht aus StructFTFileHeader.count Segmenten, die folgenden Aufbau haben.
//   + Zuerst kommt die Sturktur des Fingertemplates je nach StructFTFileHeader.type
//   + Darauf folgt ein 16bit CRC der sich über diese Struktur berechnet.
struct StructFTFileHeader
{
    unsigned short Id;    // Kennung der Datei: muss immer 0xDFDF sein.
    unsigned short offs;  // Offset auf den Datenbeginn, normalerweise Strukturgröße, falls Erweiterung nötig.
    unsigned short count; // Anzahl enthaltener Fingertemplates (zur Sicherheit mit aufgenommen)
    unsigned char type;   // Templatetyp. 0=>DIN V66400 (161Byte), 1=>Idencom-Compact (216Byte)
    unsigned short crc;   // 16bit CRC über die Headerstruktur (ohne diesen letzten 16bit Wert)
};

class Channel4 : public Datafox::Communication::Module_table, public Channel4Buffer
{
public:
    Channel4( int connection_id = Datafox::Communication::Connection_base::dv_connection_id, unsigned char device_address = Datafox::Communication::Protocol_base::dv_device_address );

    virtual ~Channel4();

    /// Schliessen der Schnittstelle.
    virtual int close_connection()
    {
        int res = 0;
        // Ist die Verbindung im Active-Mode?
        if ( get_active_mode_state() >= Datafox::Communication::ams_active )
        {
            // Beendigung sicherstellen, auch wenn er bereits beendet wurde.
            if ( m_communication_thread.joinable() )
            {
                // WorkerThread fuer Socket beenden.
                set_communication_thread_stop_signal();
                m_communication_thread.join();
            }

            res = 1;
        }
        else if ( get_connection().is_open() )
        {
            res = Module_table::close_connection();
        }

        return res;
    }

    // Verbindungsannahme von uebergebenem Listen-Socket
    int connect( SOCKET* listen_socket, int connection_id, unsigned char device_address = Datafox::Communication::Protocol_base::dv_device_address, int timeout = 3000, int alive_timeout = 10, int info_flags = 0 );

    // Aktuelle Info über den Verbundenen Teilnehmen abrufen.
    bool get_active_connection_info( char* info, int* info_length );

    // Main - Funktion fuer Befehlsausfuehrung in Workerthread
public:
    tcpipInfo m_tcpip_info;
    int m_notify;
protected:
    volatile bool m_communication_thread_start_signal;
    void set_communication_thread_start_signal( bool signal = false );
    bool get_communication_thread_start_signal();

    volatile bool m_communication_thread_stop_signal;
    void set_communication_thread_stop_signal( bool signal = true );
    bool get_communication_thread_stop_signal();

    volatile int m_communication_thread_error;
    void set_communication_thread_error( int error );
    int get_communication_thread_error();

    std::thread m_communication_thread;
    void get_communication_thread();

    // Socket fuer connect
    SOCKET *m_listen_socket;
    int m_alive_timeout;
    int m_info_flags;

public:
    // Ausfuehren eines Protokollbefehls
    virtual int call( uint16_t context, uint16_t module, uint16_t func, uint8_t *buffer, uint16_t &count, int context_value = -1 );

public:
    // Beginn und Ende der Kommunikation ueber einen TimeboySlot
    int set_timeboy_slot_state( unsigned char device_address, int state, int command_result = IV_ERROR_SUCCESS )
    {
        static clock_t stamp = 0;
        int res;
        if ( (res = get_protocol().set_timeboy_slot_state( device_address, state, command_result )) == IV_ERROR_SUCCESS )
        {
            // TimeboyIV zur Kommunikationsinitialisiserung einen Ping senden
            if ( (state && ((clock() - stamp) / CLOCKS_PER_SEC > 3)) || get_protocol().get_packet_number() == 0 )
            {
                if ( (res = ping( device_address, nullptr, nullptr )) == IV_ERROR_SUCCESS )
                {
                    stamp = clock();
                }
            }
        }

        // Wenn es kein TimeboySlot ist, dann trotzdem OK melden.
        if ( res == -IV_ERROR_SLOT_NONE )
        {
            res = IV_ERROR_SUCCESS;
        }

        return res;
    }

public:
    int generate_fingerprint_id_list( unsigned char device_address, int flags, unsigned int &record_count );
    int fingerprintBackup( unsigned char device_address, const std::string &file_path, int flags );
    int fingerprintRestore( unsigned char device_address, const std::string &file_path, int flags );

#define OFF_EXTERNAL_READER         0x00000001 // External Reader options 0=GIS, 1=PHG
#define OFF_IOMODULE                0x00000002 // IOModul
#define OFF_TBADAPTION              0x00000004 // TBAdaption
#define OFF_COMSERVER               0x00000008 // ComServer
#define OFF_TPWRITE                 0x00000010 // TransponderWrite
#define OFF_HTTP                    0x00000020 // Http-Protokoll
#define OFF_BIOKEY_UPDATE           0x00000040 // UpdateBiokey
#define OFF_SO_IOCONTROL            0x00000080 // SetupOption: Signalverarbeitung
#define OFF_KEEP_SETUP_AFTER_UPDATE 0x00000100 // Setup und Listen nach Update beibehalten
    int setOptionFirmware( unsigned char device_address, int mask, int options );
    int getOptionFirmware( unsigned char device_address, int &mask, int &options );

    int m_load_as_default_language; // Nachfolgende Uebertragung der Textdaten als Default.lng
    int setTextDataWithTimeboySlot( unsigned char device_address, const std::string &file_path );
    int setTextData( unsigned char device_address, const std::string &file_path );
    int setTouchKeyDataWithTimeboySlot( unsigned char device_address, const std::string &file_path );
    int setTouchKeyData( unsigned char device_address, const std::string &file_path );
    int setDisplayColorsWithTimeboySlot( unsigned char device_address, const std::string &file_path );
    int setDisplayColors( unsigned char device_address, const std::string &file_path );

    int setWlanConfigDataWithTimeboySlot( unsigned char device_address, const std::string &file_path );
    int setWlanConfigData( unsigned char device_address, const std::string &file_path );
    int getWlanConfigDataWithTimeboySlot( unsigned char device_address, const std::string &file_path );
    int getWlanConfigData( unsigned char device_address, const std::string &file_path );

    int setHardwareConfigDataWithTimeboySlot( unsigned char device_address, const std::string &file_path );
    int setHardwareConfigData( unsigned char device_address, const std::string &file_path );
    int getHardwareConfigDataWithTimeboySlot( unsigned char device_address, const std::string &file_path );
    int getHardwareConfigData( unsigned char device_address, const std::string &file_path );

    int writeFileDataWithTimeboySlot( unsigned char device_address, const std::string &file_path );
    int writeFileData( unsigned char device_address, const std::string &file_path );
    int readFileDataWithTimeboySlot( unsigned char device_address, const std::string &file_path );
    int readFileData( unsigned char device_address, const std::string &file_path );

private:
    // Dateiarchiv fuer Geraetedaten (Firmware) verwalten
    FileDeviceZip m_file_device;

public:
    int getTablesWithTimeboySlot( unsigned char device_address, const std::string &type_name );
    int getTables( unsigned char device_address, const std::string &type_name );

    int writeSetupWithTimeboySlot( unsigned char device_address, const std::string &file_path );
    int writeSetupData( unsigned char device_address, const std::string &file_path );
    int writeSetupData4( unsigned char device_address );
    int writeSetupData4_2( unsigned char device_address );

    int readSetupWithTimeboySlot( unsigned char device_address, const std::string &file_path );
    int readSetupData( unsigned char device_address, const std::string &file_path );
    int readSetupData4( unsigned char device_address );
    int readSetupData4_2( unsigned char device_address );

    int writeListWithTimeboySlot( unsigned char device_address );
    int writeListData( unsigned char device_address );
    int writeListData4( unsigned char device_address );

    int writeEntrance2ListWithTimeboySlot( unsigned char device_address, unsigned char list_number );
    int writeEntrance2ListData4( unsigned char device_address, unsigned char list_number );
    int writeEntrance2List4( unsigned char device_address, unsigned char list_number );

    int writeTimeboyListWithTimeboySlot( unsigned char device_address, unsigned char id, unsigned char list_number, int reserve );
    int writeTimeboyList4( unsigned char device_address, unsigned char id, unsigned char list_number, int reserve );
    int writeTimeboyListData4( unsigned char device_address, unsigned char id, unsigned char list_number, int reserve );

    std::string m_hardware_info_as_text; // Puffer fuer die Hardwareinfo in Textform.
    int writeDeviceWithTimeboySlot( unsigned char device_address, std::string &file_path );
    int writeDevice( unsigned char device_address, std::string &file_path );
    int writeDevice4( unsigned char device_address );

#pragma pack(push, 1)
    struct upload_module_signatur 
    {
        unsigned char version;
        unsigned char type;
        unsigned short flags;
        unsigned int crc32;
    } *m_upload_module_signatur;
#pragma pack(pop)
    int writeModuleWithTimeboySlot(unsigned char device_address, int type, const std::string &file_path);
    int writeModule(unsigned char device_address, int type, const std::string &file_path);

    int getVersionFirmwareFromFile( const std::string &file_path, std::string &version );

    int writeMemToFile( unsigned char device_address, int type, int address, int size, const std::string &file_path );
    int writeFileToMem( unsigned char device_address, const std::string &file_path );

    int checkData( unsigned char device_address );

    // Überlagerungen aus Channel4ModuleApplication
    int read_record( uint8_t device_address, uint8_t *buffer, std::size_t &count, uint8_t &status );
    int quit_record( uint8_t device_address, uint8_t &status );
    int clear_records( unsigned char device_address );
    int restore_records( unsigned char device_address );
    int write_memo_to_display( unsigned char device_address, unsigned char maschine, unsigned char time, unsigned char font, unsigned char audio, short flags, const std::string &text );
    int write_message_to_display( unsigned char device_address, unsigned char time, unsigned char action, unsigned char audio, const std::string &text );
    int write_variable( unsigned char device_address, const std::string &name, const std::string &val );
    int read_variable( unsigned char device_address, const std::string &name, std::string &val );
    int write_font_type( unsigned char device_address, unsigned char type );

    // Überlagerung aus Channel4ModuleSystem
    int read_system_record( uint8_t device_address, uint8_t *buffer, std::size_t &count, uint8_t &status );
    int quit_system_record( uint8_t device_address, uint8_t &status );
    int restore_system_records( unsigned char device_address );
    int read_date_time( unsigned char device_address, char* time );
    int write_date_time( unsigned char device_address, char* time );
    int read_firmware_version( unsigned char device_address, std::string &version );
    int system_reset( unsigned char device_address, unsigned short modus );
    int read_serial_number( unsigned char device_address, unsigned int &serial );
    int write_relay_state( unsigned char device_address, const std::string &name, unsigned char bus_number, unsigned char relay_number, unsigned char status, unsigned short time );
    int read_relay_state( unsigned char device_address, const std::string &name, unsigned char bus_number, unsigned char relay_number, unsigned char &status, unsigned short &time );
    int write_system_variable( unsigned char device_address, const std::string &name, const std::string &val );
    int read_system_variable( unsigned char device_address, const std::string &name, std::string &val );
    int write_virtual_key( unsigned char device_address, unsigned char key_code, unsigned char flags );

    // Informationen über das Gerät sammeln.
    int read_device_info( unsigned char device_address, DeviceInfo &info );
    // Informationen über die verbaute Hardware im Gerät sammeln.
    int read_hardware_info( unsigned char device_address, HardwareInfo &info );
    // Informationen über die Enthaltene Firmware.
    int read_software_info( unsigned char device_address, SoftwareInfo &info );
    // Informationen über die Zusatzoptionen der Firmware.
    int read_firmware_options( unsigned char device_address, FirmwareOptions &options );
    // Setzen der Zusatzoptionen der Firmware.
    int write_firmware_options( unsigned char device_address, const FirmwareOptions &options );

    // Daten getunnelt über Docking schicken.
    int enter_transparency_mode( unsigned char device_address, unsigned short id, unsigned short flags, unsigned int read_buffer_size, unsigned int write_buffer_size, int timeout );
    int exit_transparency_mode( unsigned char device_address );
    bool is_transparency_mode();

    // Gelbe LED bei Dockingstation-V2 setzen/lesen
    int write_led_state( unsigned char device_address, unsigned slot, unsigned char id, unsigned char state);
    int read_led_state( unsigned char device_address, unsigned slot, unsigned char id, unsigned char &state);
};

#endif
